@@ -31,6 +31,7 @@
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 
+#include "yelp-uri.h"
 #include "yelp-error.h"
 #include "yelp-toc-pager.h"
 
@@ -86,7 +87,7 @@ struct _TOC {
     GSList      *subtocs;
     GSList      *subomfs;
 
-    gboolean     pruned;
+    gboolean     has_subtocs;
 };
 
 static void          toc_pager_class_init      (YelpTocPagerClass *klass);
@@ -198,7 +199,11 @@ toc_pager_dispose (GObject *object)
 void
 yelp_toc_pager_init (void)
 {
-    toc_pager = (YelpTocPager *) g_object_new (YELP_TYPE_TOC_PAGER, NULL);
+    YelpURI *uri;
+    
+    uri = yelp_uri_new ("toc:");
+    toc_pager = (YelpTocPager *) g_object_new (YELP_TYPE_TOC_PAGER, 
+    					       "uri", uri, NULL);
 
     yelp_pager_start (YELP_PAGER (toc_pager));
 }
@@ -240,7 +245,7 @@ toc_pager_process (YelpPager *pager)
     YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
 
     priv->dir_pending = g_slist_prepend (priv->dir_pending,
-					 DATADIR "/omf");
+					 g_strdup (DATADIR "/omf"));
 
     priv->pending_func = (ProcessFunction) process_dir_pending;
 
@@ -364,8 +369,10 @@ process_dir_pending (YelpTocPager *pager)
     }
 
  done:
-    gnome_vfs_file_info_unref (file_info);
-    gnome_vfs_directory_close (dir);
+    if (file_info)
+    	gnome_vfs_file_info_unref (file_info);
+    if (dir && !result)
+	gnome_vfs_directory_close (dir);
     g_slist_free_1 (first);
     g_free (dirstr);
 
@@ -378,8 +385,8 @@ process_dir_pending (YelpTocPager *pager)
 static gboolean
 process_omf_pending (YelpTocPager *pager)
 {
-    GSList    *first;
-    gchar     *file;
+    GSList    *first = NULL;
+    gchar     *file  = NULL;
     xmlDocPtr  omf_doc = NULL;
     xmlNodePtr omf_cur;
     gint       lang_priority;
@@ -390,6 +397,9 @@ process_omf_pending (YelpTocPager *pager)
     YelpTocPagerPriv *priv  = YELP_TOC_PAGER (pager)->priv;
 
     first = priv->omf_pending;
+    if (!first)
+	goto done;
+
     priv->omf_pending = g_slist_remove_link (priv->omf_pending, first);
 
     file = (gchar *) first->data;
@@ -659,30 +669,22 @@ process_toc_pending (YelpTocPager *pager)
 
 		// FIXME: insert_sorted
 		for ( ; omf; omf = omf->next)
-		    omfs = g_slist_prepend (omfs, omf->data);
+		    toc->subomfs = g_slist_prepend (toc->subomfs, omf->data);
 
 		xmlFree (cat);
 	    }
     }
 
-    // Determine whether to prune this TOC page.
-    toc->pruned = TRUE;
-    if (omfs) {
-	toc->pruned  = FALSE;
-	toc->subomfs = omfs;
-    } else {
-	for (c = toc->subtocs; c; c = c->next) {
-	    cur     = (xmlNodePtr) c->data;
-	    cur_toc = (TOC *) cur->_private;
+    toc->has_subtocs = FALSE;
+    for (c = toc->subtocs; c; c = c->next) {
+	cur     = (xmlNodePtr) c->data;
+	cur_toc = (TOC *) cur->_private;
 
-	    if (cur_toc && !cur_toc->pruned) {
-		toc->pruned = FALSE;
-		break;
-	    }
-	}
+	if (cur_toc && (cur_toc->has_subtocs || cur_toc->subomfs))
+	    toc->has_subtocs = TRUE;
     }
 
-    if (!toc->pruned) {
+    if (toc->has_subtocs || toc->subomfs) {
 	gchar *page = toc_write_page (toc);
 	yelp_pager_add_page (YELP_PAGER (pager),
 			     toc->id,
@@ -774,7 +776,7 @@ toc_write_page (TOC    *toc)
     strs[i] = g_strconcat ("<html><body><h1>", toc->title, "</h1>\n", NULL);
     page_len += strlen (strs[i]);
 
-    if (toc->subtocs) {
+    if (toc->has_subtocs) {
 	strs[++i] = g_strconcat ("<h2>", _("Categories"), "</h2><ul>\n", NULL);
 	page_len += strlen (strs[i]);
 
@@ -782,7 +784,7 @@ toc_write_page (TOC    *toc)
 	    xmlNodePtr  cur     = (xmlNodePtr) c->data;
 	    TOC        *cur_toc = (TOC *) cur->_private;
 
-	    if (cur_toc && !cur_toc->pruned) {
+	    if (cur_toc && (cur_toc->has_subtocs || cur_toc->subomfs)) {
 		strs[++i] = g_strconcat
 		    ("<li><a href='toc:", cur_toc->id, "'>",
 		     cur_toc->title,
